@@ -9,32 +9,30 @@ psm_plot <- function(psm_result,
                      line_notcheap = TRUE,
                      line_notexpensive = TRUE,
                      point_idp = TRUE,
-                     point_color_idp = "#009E73",
+                     point_color_idp = get_psm_constant("DEFAULT_COLORS.IDP"),
                      label_idp = TRUE,
                      point_opp = TRUE,
-                     point_color_opp = "#009E73",
+                     point_color_opp = get_psm_constant("DEFAULT_COLORS.OPP"),
                      label_opp = TRUE,
-                     pricerange_color = "grey50",
-                     pricerange_alpha = 0.3,
-                     line_color = c(
-                       "too cheap" = "#009E73",
-                       "not cheap" = "#009E73",
-                       "not expensive" = "#D55E00",
-                       "too expensive" = "#D55E00"
-                     ),
-                     line_type = c(
-                       "too cheap" = "dotted",
-                       "not cheap" = "solid",
-                       "not expensive" = "solid",
-                       "too expensive" = "dotted"
-                     )) {
-  # ensure that only objects that are result of psm_analysis/psm_analysis_weighted are accepted as input
+                     pricerange_color = get_psm_constant("DEFAULT_COLORS.PRICE_RANGE"),
+                     pricerange_alpha = get_psm_constant("DEFAULT_PLOT_SETTINGS.PRICE_RANGE_ALPHA"),
+                     line_color = get_psm_constant("DEFAULT_COLORS.LINE_COLORS"),
+                     line_type = get_psm_constant("DEFAULT_LINE_TYPES")) {
+
+  # Ensure that only objects that are result of psm_analysis are accepted as input
   stopifnot(class(psm_result) == "psm")
-
-  # base of plot
+  
+  # Get plot settings from constants
+  line_size <- get_psm_constant("DEFAULT_PLOT_SETTINGS.LINE_SIZE")
+  point_size_idp <- get_psm_constant("DEFAULT_PLOT_SETTINGS.POINT_SIZE_IDP")
+  point_size_opp <- get_psm_constant("DEFAULT_PLOT_SETTINGS.POINT_SIZE_OPP")
+  point_shape_idp <- get_psm_constant("DEFAULT_PLOT_SETTINGS.POINT_SHAPE_IDP")
+  point_shape_opp <- get_psm_constant("DEFAULT_PLOT_SETTINGS.POINT_SHAPE_OPP")
+  
+  # Base of plot
   plot_object <- ggplot2::ggplot(data = psm_result$data_vanwestendorp, ggplot2::aes(x = .data$price))
-
-  # shaded area for acceptable pricerange
+  
+  # Shaded area for acceptable pricerange
   if (isTRUE(shade_pricerange)) {
     plot_object <- plot_object + ggplot2::annotate(
       geom = "rect",
@@ -44,107 +42,123 @@ psm_plot <- function(psm_result,
       fill = pricerange_color, alpha = pricerange_alpha
     )
   }
-
-  # line for "too cheap"
+  
+  # Line for "too cheap"
   if (isTRUE(line_toocheap)) {
     plot_object <- plot_object + ggplot2::geom_line(ggplot2::aes(
       y = .data$ecdf_toocheap,
       colour = "too cheap",
       linetype = "too cheap"
     ),
-    size = 1
+    linewidth = line_size
     )
   }
-
-  # line for "too expensive"
+  
+  # Line for "too expensive"
   if (isTRUE(line_tooexpensive)) {
     plot_object <- plot_object + ggplot2::geom_line(ggplot2::aes(
       y = .data$ecdf_tooexpensive,
       colour = "too expensive",
       linetype = "too expensive"
     ),
-    size = 1
+    linewidth = line_size
     )
   }
-
-  # line for "not cheap"
+  
+  # Line for "not cheap"
   if (isTRUE(line_notcheap)) {
     plot_object <- plot_object + ggplot2::geom_line(ggplot2::aes(
       y = .data$ecdf_not_cheap,
       colour = "not cheap",
       linetype = "not cheap"
     ),
-    size = 1
+    linewidth = line_size
     )
   }
-
-  # line for "not expensive"
+  
+  # Line for "not expensive"
   if (isTRUE(line_notexpensive)) {
     plot_object <- plot_object + ggplot2::geom_line(ggplot2::aes(
       y = .data$ecdf_not_expensive,
       colour = "not expensive",
       linetype = "not expensive"
     ),
-    size = 1
+    linewidth = line_size
     )
   }
-
-
-  # point for Indifference Price Point (intersection of "cheap" and "expensive")
+  
+  # Point for Indifference Price Point (intersection of "cheap" and "expensive")
   if (isTRUE(point_idp)) {
+    # Use interpolation to find y-coordinate at exact IDP price
+    y_idp <- approx(x = psm_result$data_vanwestendorp$price, 
+                    y = psm_result$data_vanwestendorp$ecdf_not_cheap, 
+                    xout = psm_result$idp)$y
+    
     plot_object <- plot_object + ggplot2::annotate(
       geom = "point",
       x = psm_result$idp,
-      y = psm_result$data_vanwestendorp$ecdf_not_cheap[psm_result$data_vanwestendorp$price == psm_result$idp],
-      size = 5,
-      shape = 18,
+      y = y_idp,
+      size = point_size_idp,
+      shape = point_shape_idp,
       colour = point_color_idp
     )
   }
-
-  # point for Optimal Price Point (intersection of "too cheap" and "too expensive")
-  if (isTRUE(point_idp)) {
+  
+  # Point for Optimal Price Point (intersection of "too cheap" and "too expensive")
+  if (isTRUE(point_opp)) {
+    # Use interpolation to find y-coordinate at exact OPP price
+    y_opp <- approx(x = psm_result$data_vanwestendorp$price, 
+                    y = psm_result$data_vanwestendorp$ecdf_toocheap, 
+                    xout = psm_result$opp)$y
+    
     plot_object <- plot_object + ggplot2::annotate(
       geom = "point",
       x = psm_result$opp,
-      y = psm_result$data_vanwestendorp$ecdf_toocheap[psm_result$data_vanwestendorp$price == psm_result$opp],
-      size = 3,
-      shape = 17,
+      y = y_opp,
+      size = point_size_opp,
+      shape = point_shape_opp,
       colour = point_color_opp
     )
   }
-
-
+  
   # Adding line color and line style
-
   plot_object <- plot_object + ggplot2::scale_colour_manual(name = "Legend", values = line_color)
   plot_object <- plot_object + ggplot2::scale_linetype_manual(name = "Legend", values = line_type)
-
+  
   # Adding label for Indifference Price Point
-
   if (isTRUE(label_idp)) {
+    # Use interpolation to find y-coordinate at exact IDP price for label
+    y_idp_label <- approx(x = psm_result$data_vanwestendorp$price, 
+                          y = psm_result$data_vanwestendorp$ecdf_not_cheap, 
+                          xout = psm_result$idp)$y
+    
     plot_object <- plot_object + ggplot2::annotate(
       geom = "label",
       x = psm_result$idp,
-      y = psm_result$data_vanwestendorp$ecdf_not_cheap[psm_result$data_vanwestendorp$price == psm_result$idp],
+      y = y_idp_label,
       label = paste("IDP: ", format(psm_result$idp, nsmall = 2)),
       fill = "white",
       alpha = 0.5
     )
   }
-
-
+  
+  # Adding label for Optimal Price Point
   if (isTRUE(label_opp)) {
+    # Use interpolation to find y-coordinate at exact OPP price for label
+    y_opp_label <- approx(x = psm_result$data_vanwestendorp$price, 
+                          y = psm_result$data_vanwestendorp$ecdf_toocheap, 
+                          xout = psm_result$opp)$y
+    
     plot_object <- plot_object + ggplot2::annotate(
       geom = "label",
       x = psm_result$opp,
-      y = psm_result$data_vanwestendorp$ecdf_toocheap[psm_result$data_vanwestendorp$price == psm_result$opp],
+      y = y_opp_label,
       label = paste("OPP: ", format(psm_result$opp, nsmall = 2)),
       fill = "white",
       alpha = 0.5
     )
   }
-
-  # return the resulting ggplot object
+  
+  # Return the resulting ggplot object
   return(plot_object)
 }
